/*
 *
 * The  method to do MSBIN -> LONG is
 * Keep shifting the initial value to the left until value & 0x800000 is true
 * (i.e. bit 23 is set). Then clear bit 23, which is the 'free' bit.
 * While doing this, count how many shifts you have to do. The exponent is
 * the offset (0x80) + (24 - number of shifts). So if you have start with 2,
 * you have to shift it right 22 times to get 0x800000. AND'ing this
 * with ~0x800000 gives zero, and the exponent is 0x80 + 24 -22 = 0x82, so
 * the final value should be 0x82000000 in MSBIN format.
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include "qwk.h"

ulong msb2ul (unchar *msk)
{
    return (((msk[0] | ((ulong) msk[1] << 8) | ((ulong) msk[2] <<
                16)) | 0x800000L) >> (24 - (msk[3] - 0x80))); 
}

#if defined (QDOS) || defined (AMIGA)
ushort swapword (ushort val)
{
    return (ushort) (val << 8) + (val >> 8);
}

ulong swaplong (ulong val)
{
    return (ulong) (((ulong) swapword (val & 0xFFFF) << 16) |
		    (ulong) swapword (val >> 16));
}
#endif

ulong ul2msb(ulong n)
{
    ulong j;
    short i;

    if(n == 0)
    {
        j = 0;
    }
    else
    {
        j = n;
        for(i = 0; !(j & 0x800000L); i++)
        {
            j  <<= 1;
        }
        j &= ~0x800000L;
        j |= (0x80 + 24 - i) << 24;
    }
#if defined(QDOS) || defined(AMIGA)
    return swaplong(j);
#else
    return j;
#endif
}

#ifdef TEST
/*
 * av[1] should be a QWK *.NDX file.
 * Verifies MSBIN <-> 32bit int conversion
 *   $ msb test.ndx
 *   $ cmp test.ndx dup.inx 
 * (DOS etc must open the files as binary (rb,wb)).
 */

main(int ac, char **av)
{
    FILE *fp;
    unchar msb[5];
    ulong z, z1;
    
    if(fp = fopen(av[1], "r"))
    {
	FILE *op = fopen("dup.inx", "w");
	
        while(fread(msb, 1, 5, fp))
        {
            z = msb2ul (msb);
            printf("%02x%02x%02x%02x gives %4d (%d) ",
                    msb[3], msb[2], msb[1], msb[0], z, msb[4]);

            z1 = ul2msb(z);

            *(ulong *)msb = z1; 
 
            printf("Go back gives %02x%02x%02x%02x\n",
                        msb[3], msb[2], msb[1], msb[0]);

	    fwrite(msb, 1, 5, op);
        }
        fclose(fp);
    }
    return 0;
}
#endif
